@@ -9,6 +9,7 @@ __all__ = [
     'BytesFieldMarshaller',
     'CollectionFieldMarshaller',
     'DateTimeFieldMarshaller',
+    'DictFieldMarshaller',
     'FloatFieldMarshaller',
     'IntFieldMarshaller',
     'ObjectLookupFieldMarshaller',
@@ -112,7 +113,7 @@ class URLDereferencingMixin:
         path_parts = [urllib.unquote(part) for part in path.split('/')]
         path_parts.pop(0)
         path_parts.reverse()
-        request = config.createRequest(StringIO(), {'PATH_INFO' : path})
+        request = config.createRequest(StringIO(), {'PATH_INFO': path})
         request.setTraversalStack(path_parts)
         root = request.publication.getApplication(self.request)
         return request.traverse(root)
@@ -125,7 +126,6 @@ class SimpleFieldMarshaller:
     """
     implements(IFieldMarshaller)
 
-
     # Set this to type or tuple of types that the JSON value must be of.
     _type = None
 
@@ -153,7 +153,7 @@ class SimpleFieldMarshaller:
         if value != '':
             try:
                 v = value
-                if isinstance (v, str):
+                if isinstance(v, str):
                     v = v.decode('utf8') # assume utf8
                 # XXX gary 2009-03-28
                 # The use of the enclosing brackets is a hack to work around
@@ -327,7 +327,7 @@ class FixedVocabularyFieldMarshaller(SimpleFieldMarshaller):
         """Describe all values, not just the selected value."""
         unmarshalled = []
         for item in self.field.vocabulary:
-            item_dict = {'token' : item.token, 'title' : item.title}
+            item_dict = {'token': item.token, 'title': item.title}
             if value.title == item.title:
                 item_dict['selected'] = True
             unmarshalled.append(item_dict)
@@ -462,6 +462,84 @@ class SetFieldMarshaller(AbstractCollectionFieldMarshaller):
         return set
 
 
+class DictFieldMarshaller(SimpleFieldMarshaller):
+    """A marshaller for Dicts.
+
+    It looks up the marshallers for its (key-type, value-type), to handle its
+    contained name/value pairs.
+    """
+    _type = dict
+    _type_error_message = 'not a dict: %r'
+
+    def __init__(self, field, request):
+        """See `SimpleFieldMarshaller`.
+
+        This also looks for the appropriate marshaller for key_type and
+        value_type.
+        """
+        super(DictFieldMarshaller, self).__init__(
+            field, request)
+        self.key_marshaller = getMultiAdapter(
+            (field.key_type, request), IFieldMarshaller)
+        self.value_marshaller = getMultiAdapter(
+            (field.value_type, request), IFieldMarshaller)
+
+    def _marshall_from_request(self, value):
+        """See `IFieldMarshaller`.
+
+        All items in the dict are marshalled using the appropriate
+        `IFieldMarshaller` for the key_type, value_type.
+
+        When coming in via HTML request parameters, the dict is sent as a list
+        of comma separated "key,value" strings.
+        If the value isn't a list or dict, transform it into a one-element list.
+        That allows web client to submit one-element list of strings
+        without having to JSON-encode it.
+        """
+        if not isinstance(value, list) and not isinstance(value, dict):
+            value = [value]
+        if isinstance(value, list):
+            try:
+                value = dict(nv.split(',', 2) for nv in value)
+            except ValueError:
+                raise ValueError(
+                    u"got '%s', list of name,value pairs" % value)
+        return dict(
+            (self.key_marshaller.marshall_from_json_data(key),
+            self.value_marshaller.marshall_from_json_data(val))
+            for key, val in value.items())
+
+    def _marshall_from_json_data(self, value):
+        """See `SimpleFieldMarshaller`.
+
+        Marshall the items in the dict using the appropriate
+        (key,value) marshallers.
+
+        The incoming value may be a list of name,value pairs in which case
+        we convert to a dict first.
+        """
+        if isinstance(value, list):
+            value = dict(value)
+        # Call super to check for errors and raise a ValueError if necessary.
+        value = super(
+            DictFieldMarshaller, self)._marshall_from_json_data(value)
+        return dict(
+            (self.key_marshaller.marshall_from_json_data(key),
+            self.value_marshaller.marshall_from_json_data(val))
+            for key, val in value.items())
+
+    def unmarshall(self, entry, value):
+        """See `SimpleFieldMarshaller`.
+
+        The value is unmarshalled into a dict and all its items are
+        unmarshalled using the appropriate FieldMarshaller.
+        """
+        return dict(
+            (self.key_marshaller.unmarshall(entry, key),
+            self.value_marshaller.unmarshall(entry, val))
+               for key, val in value.items())
+
+
 class CollectionFieldMarshaller(SimpleFieldMarshaller):
     """A marshaller for collection fields."""
     implements(IUnmarshallingDoesntNeedValue)
@@ -569,4 +647,3 @@ class ObjectLookupFieldMarshaller(SimpleFieldMarshaller,
         # to the object underlying a resource, we need to strip its
         # security proxy.
         return removeSecurityProxy(resource).context
-
